@@ -15,6 +15,7 @@ use mas_data_model::{
 use mas_storage::{
     Page, Pagination,
     compat::{CompatSessionFilter, CompatSessionRepository},
+    pagination::Node,
 };
 use rand::RngCore;
 use sea_query::{Expr, PostgresQueryBuilder, Query, enum_def};
@@ -59,6 +60,12 @@ struct CompatSessionLookup {
     last_active_ip: Option<IpAddr>,
 }
 
+impl Node<Ulid> for CompatSessionLookup {
+    fn cursor(&self) -> Ulid {
+        self.compat_session_id.into()
+    }
+}
+
 impl From<CompatSessionLookup> for CompatSession {
     fn from(value: CompatSessionLookup) -> Self {
         let id = value.compat_session_id.into();
@@ -106,6 +113,12 @@ struct CompatSessionAndSsoLoginLookup {
     compat_sso_login_exchanged_at: Option<DateTime<Utc>>,
 }
 
+impl Node<Ulid> for CompatSessionAndSsoLoginLookup {
+    fn cursor(&self) -> Ulid {
+        self.compat_session_id.into()
+    }
+}
+
 impl TryFrom<CompatSessionAndSsoLoginLookup> for (CompatSession, Option<CompatSsoLogin>) {
     type Error = DatabaseInconsistencyError;
 
