@@ -12,6 +12,7 @@ use mas_data_model::{
 };
 use mas_storage::{
     Page, Pagination,
+    pagination::Node,
     user::{UserEmailFilter, UserEmailRepository},
 };
 use rand::RngCore;
@@ -51,6 +52,12 @@ struct UserEmailLookup {
     created_at: DateTime<Utc>,
 }
 
+impl Node<Ulid> for UserEmailLookup {
+    fn cursor(&self) -> Ulid {
+        self.user_email_id.into()
+    }
+}
+
 impl From<UserEmailLookup> for UserEmail {
     fn from(e: UserEmailLookup) -> UserEmail {
         UserEmail {
